function [IRF, VAR] = IRFcho(VAR,VARopt)

%% Retrieve and initialize variables 
%===============================================
nsteps = VARopt.nsteps;
ident  = VARopt.ident;
Fcomp = VAR.Fcomp;
nvar  = VAR.nvar;
nlag  = VAR.nlag;
sigma = VAR.sigma;

IRF = nan(nsteps,nvar,nvar);

%% Compute the impulse response
%===============================================
for mm=1:nvar
      
    % Initialize the impulse response vector
    response = zeros(nvar, nsteps);
    
    % Create the impulse vector
    impulse = zeros(nvar,1); 
    
    % Get the matrix invA containing the structural impulses
    if strcmp(ident,'oir')
        [out, chol_flag] = chol(sigma);
        if chol_flag~=0; error('VCV is not positive definite'); end
        invA = out';
    end
    
    % Set the size of the shock
        impulse(mm,1) = 1; % one stdev shock
 
    % First period impulse response (=impulse vector)
    response(:,1) = invA*impulse;


    % Make it comparable with companion
    impulse_big  = [response(:,1)' zeros(1, nvar*(nlag-1))]';
    
    % Recursive computation of impulse response
    Fcomp_eye = eye(size(Fcomp,1)); 
    for kk = 2:nsteps
        Fcomp_eye = Fcomp_eye * Fcomp; % this is the multiplier Fcomp^n
        response_big   = Fcomp_eye * impulse_big;
        response(:,kk) = response_big(1:nvar);
    end
    IRF(:,:,mm) = response';
end

% Update VAR
VAR.invA = invA;



